/*
 * See Licensing and Copyright notice in naev.h
 */
#pragma once

#include "space.h"

#define HIDE_DEFAULT_SPOB 1.0 /**< Default hide value for new spobs. */

void sysedit_open( StarSystem *sys );
void sysedit_sysScale( StarSystem *sys, double factor );
