local mg = require "core"

love.load         = mg.load
love.keypressed   = mg.keypressed
love.mousemoved   = mg.mousemoved
love.mousepressed = mg.mousepressed
love.mousereleased = mg.mousereleased
love.draw         = mg.draw
love.update       = mg.update
