return {
   fct            = faction.get("Yetmer"),
   rep_max        = 50,
   rep_max_var    = "_fcap_yetmer",
   rep_max_tags   = {},
   hit_range      = 2,
   destroy_max    = 50,
   disable_max    = 50,
   board_max      = 50,
   capture_max    = 50,
   text = {
      [100] = _("War Hero"),
      [70]  = _("Saviour"),
      [20]  = _("Combatant"),
      [0]   = _("Neutral"),
      [-1]  = _("Enemy"),
      [-30] = _("War Criminal"),
   },
}
