<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
        <link rel="stylesheet" href="../naev-common.css" type="text/css" />
        <link rel="stylesheet" href="../naev-global.css" type="text/css" />{% block head %}{% endblock %}</head>
        <link rel="shortcut icon" href="/favicon.ico" />
    <body>
        <div id="naev-header">
            <div id="naev-top">
                <a href="http://naev.org">
                    <img src="http://naev.org/naevsmall.png" alt="Home" style="vertical-align:baseline" />
                </a>
            </div>
            <ul class="naevlinks">
                <li>
                    <a href="http://blog.naev.org/">Blog</a> &middot;
                    <a href="http://forum.naev.org/">Forum</a> &middot;
                    <a href="http://wiki.naev.org/">Wiki</a> &middot;
                    <a href="http://blog.naev.org/downloads/">Downloads</a> &middot;
                    <a href="http://api.naev.org/">Lua API</a> &middot;
                    <a href="http://db.naev.org/">Ship Database</a> &middot;
                    <a href="http://blog.naev.org/chat/">Chat</a>
                </li>
            </ul>
        </div>
        <div id="container">{% block body %}{% endblock %}</div><!-- /Container -->
        <div id="footer">Generated on {{ date }}</div>
    </body>
</html>
