{%- macro shipImage(gfx, shipName='') %}
<img width="128" height="128" src="media/{{gfx}}" alt="Image for {{shipName}}" />
{%- endmacro %}

{%- macro shipImageLarge(gfx, shipName='') %}
<img width="256" height="256" src="media/{{gfx}}" alt="Image for {{shipName}}" />
{%- endmacro %}
