local intrinsics = {}

-- Brigand (Interceptor) intrinsics
intrinsics["Soromid Brigand"] = {
   {
      name = _("Innate"),
      outfit = {
         "Perleve Cerebrum I",
         "Perlevis Cortex I",
         "Perlevis Gene Drive I",
         "Stinger Organ I",
         "Stinger Organ I",
      },
      slot = {
         "brain",
         "shell",
         "genedrive",
         "rightweap",
         "leftweap",
      },
   },
   {
      name = _("Cortex Growth"),
      outfit = "Perlevis Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth"),
      outfit = "Perlevis Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Weapon Organ Growth"),
      outfit = {
         "Stinger Organ II",
         "Stinger Organ II",
      },
      slot = {
         "rightweap",
         "leftweap",
      },
   },
   {
      name = _("Cerebrum Growth"),
      outfit = "Perleve Cerebrum II",
      slot = "brain",
   },
}

-- Marauder (Bomber) intrinsics
intrinsics["Soromid Marauder"] = {
   {
      name = _("Innate"),
      outfit = {
         "Laevum Cerebrum I",
         "Laevis Cortex I",
         "Laevis Gene Drive I",
         "Stinger Organ I",
         "Stinger Organ I",
      },
      slot = {
         "brain",
         "shell",
         "genedrive",
         "leftweap",
         "rightweap",
      },
   },
   {
      name = _("Cortex Growth"),
      outfit = "Laevis Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth"),
      outfit = "Laevis Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Weapon Organ Growth I"),
      outfit = {
         "Stinger Organ II",
         "Stinger Organ II",
      },
      slot = {
         "rightweap",
         "leftweap",
      },
   },
   {
      name = _("Cerebrum Growth"),
      outfit = "Laevum Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth II"),
      outfit = {
         "Stinger Organ III",
         "Stinger Organ III",
      },
      slot = {
         "rightweap",
         "leftweap",
      },
   },
}

-- Reaver (Fighter) intrinsics
intrinsics["Soromid Reaver"] = {
   {
      name = _("Innate"),
      outfit = {
         "Laevum Cerebrum I",
         "Laevis Cortex I",
         "Laeviter Gene Drive I",
         "Stinger Organ I",
         "Stinger Organ I",
         "Stinger Organ I",
      },
      slot = {
         "brain",
         "shell",
         "genedrive",
         "leftweap",
         "middleweap",
         "rightweap",
      },
   },
   {
      name = _("Cortex Growth"),
      outfit = "Laevis Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth"),
      outfit = "Laeviter Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Weapon Organ Growth I"),
      outfit = {
         "Stinger Organ II",
         "Stinger Organ II",
         "Stinger Organ II",
      },
      slot = {
         "rightweap",
         "middleweap",
         "leftweap",
      },
   },
   {
      name = _("Cerebrum Growth"),
      outfit = "Laevum Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth II"),
      outfit = {
         "Stinger Organ III",
         "Stinger Organ III",
         "Stinger Organ III",
      },
      slot = {
         "rightweap",
         "middleweap",
         "leftweap",
      },
   },
}

-- Odium (Corvette) intrinsics
intrinsics["Soromid Odium"] = {
   {
      name = _("Innate"),
      outfit = {
         "Mediocre Cerebrum I",
         "Mediocris Cortex I",
         "Mediocris Gene Drive I",
         "Talon Organ I",
         "Talon Organ I",
         "Talon Organ I",
      },
      slot = {
         "brain",
         "shell",
         "genedrive",
         "weap1",
         "weap2",
         "weap3",
      },
   },
   {
      name = _("Cortex Growth"),
      outfit = "Mediocris Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth I"),
      outfit = "Mediocris Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Weapon Organ Growth I"),
      outfit = {
         "Talon Organ II",
         "Talon Organ II",
         "Talon Organ II",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
      },
   },
   {
      name = _("Cerebrum Growth"),
      outfit = "Mediocre Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth II"),
      outfit = {
         "Talon Organ III",
         "Talon Organ III",
         "Talon Organ III",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
      },
   },
   {
      name = _("Gene Drive Growth II"),
      outfit = "Mediocris Gene Drive III",
      slot = "genedrive",
   },
}

-- Nyx (Destroyer) intrinsics
intrinsics["Soromid Nyx"] = {
   {
      name = _("Innate"),
      outfit = {
         "Largum Cerebrum I",
         "Largus Cortex I",
         "Largus Gene Drive I",
         "Talon Organ I",
         "Talon Organ I",
         "Talon Organ I",
      },
      slot = {
         "brain",
         "shell",
         "genedrive",
         "weap1",
         "weap2",
         "weap3",
      },
   },
   {
      name = _("Cortex Growth I"),
      outfit = "Largus Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth"),
      outfit = "Largus Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Weapon Organ Growth I"),
      outfit = {
         "Talon Organ II",
         "Talon Organ II",
         "Talon Organ II",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
      },
   },
   {
      name = _("Cerebrum Growth"),
      outfit = "Largum Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth II"),
      outfit = {
         "Talon Organ III",
         "Talon Organ III",
         "Talon Organ III",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
      },
   },
   {
      name = _("Cortex Growth II"),
      outfit = "Largus Cortex III",
      slot = "shell",
   },
   {
      name = _("Weapon Organ Growth III"),
      outfit = {
         "Talon Organ IV",
         "Talon Organ IV",
         "Talon Organ IV",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
      },
   },
}

-- Copia (Bulk Freighter) intrinsics
intrinsics["Soromid Copia"] = {
   {
      name = _("Innate"),
      outfit = {
         "Grandis Gene Drive I",
         "Ponderosus Cortex I",
         "Ponderosum Cerebrum I",
         "Tentacle Organ I",
         "Tentacle Organ I",
      },
      slot = {
         "genedrive",
         "shell",
         "brain",
         "weap1",
         "weap2",
      },
   },
   {
      name = _("Cortex Growth I"),
      outfit = "Ponderosus Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth I"),
      outfit = "Grandis Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Cerebrum Growth I"),
      outfit = "Ponderosum Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth"),
      outfit = {
         "Tentacle Organ II",
         "Tentacle Organ II",
      },
      slot = {
         "weap1",
         "weap2",
      },
   },
   {
      name = _("Cortex Growth II"),
      outfit = "Ponderosus Cortex III",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth II"),
      outfit = "Grandis Gene Drive III",
      slot = "genedrive",
   },
   {
      name = _("Cerebrum Growth II"),
      outfit = "Ponderosum Cerebrum III",
      slot = "brain",
   },
   {
      name = _("Cortex Growth III"),
      outfit = "Ponderosus Cortex IV",
      slot = "shell",
   },
}

-- Ira (Cruiser) intrinsics
intrinsics["Soromid Ira"] = {
   {
      name = _("Innate"),
      outfit = {
         "Ponderosus Gene Drive I",
         "Ponderosus Cortex I",
         "Ponderosum Cerebrum I",
         "Tentacle Organ I",
         "Tentacle Organ I",
         "Tentacle Organ I",
         "Tentacle Organ I",
      },
      slot = {
         "genedrive",
         "shell",
         "brain",
         "weap1",
         "weap2",
         "weap3",
         "weap4",
      },
   },
   {
      name = _("Cortex Growth I"),
      outfit = "Ponderosus Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth I"),
      outfit = "Ponderosus Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Cerebrum Growth I"),
      outfit = "Ponderosum Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth"),
      outfit = {
         "Tentacle Organ II",
         "Tentacle Organ II",
         "Tentacle Organ II",
         "Tentacle Organ II",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
         "weap4",
      },
   },
   {
      name = _("Cortex Growth II"),
      outfit = "Ponderosus Cortex III",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth II"),
      outfit = "Ponderosus Gene Drive III",
      slot = "genedrive",
   },
   {
      name = _("Cerebrum Growth II"),
      outfit = "Ponderosum Cerebrum III",
      slot = "brain",
   },
   {
      name = _("Cortex Growth III"),
      outfit = "Ponderosus Cortex IV",
      slot = "shell",
   },
}

-- Vox (Battleship) Intrinsics
intrinsics["Soromid Vox"] = {
   {
      name = _("Innate"),
      outfit = {
         "Immanis Gene Drive I",
         "Immanis Cortex I",
         "Immane Cerebrum I",
         "Tentacle Organ I",
         "Tentacle Organ I",
         "Tentacle Organ I",
         "Tentacle Organ I",
      },
      slot = {
         "genedrive",
         "shell",
         "brain",
         "weap1",
         "weap2",
         "weap3",
         "weap4",
      },
   },
   {
      name = _("Cortex Growth I"),
      outfit = "Immanis Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth I"),
      outfit = "Immanis Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Weapon Organ Growth I"),
      outfit = {
         "Tentacle Organ II",
         "Tentacle Organ II",
         "Tentacle Organ II",
         "Tentacle Organ II",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
         "weap4",
      },
   },
   {
      name = _("Cerebrum Growth I"),
      outfit = "Immane Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Cortex Growth II"),
      outfit = "Immanis Cortex III",
      slot = "shell",
   },
   {
      name = _("Weapon Organ Growth II"),
      outfit = {
         "Tentacle Organ III",
         "Tentacle Organ III",
         "Tentacle Organ III",
         "Tentacle Organ III",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
         "weap4",
      },
   },
   {
      name = _("Gene Drive Growth II"),
      outfit = "Immanis Gene Drive III",
      slot = "genedrive",
   },
   {
      name = _("Cerebrum Growth II"),
      outfit = "Immane Cerebrum III",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth III"),
      outfit = {
         "Tentacle Organ IV",
         "Tentacle Organ IV",
         "Tentacle Organ IV",
         "Tentacle Organ IV",
      },
      slot = {
         "weap1",
         "weap2",
         "weap3",
         "weap4",
      },
   },
}

-- Arx (Carrier) Intrinsics
intrinsics["Soromid Arx"] = {
   {
      name = _("Innate"),
      outfit = {
         "Magnus Gene Drive I",
         "Immanis Cortex I",
         "Immane Cerebrum I",
         "Tentacle Organ I",
         "Tentacle Organ I",
      },
      slot = {
         "genedrive",
         "shell",
         "brain",
         "rightweap",
         "leftweap",
      },
   },
   {
      name = _("Cortex Growth I"),
      outfit = "Immanis Cortex II",
      slot = "shell",
   },
   {
      name = _("Gene Drive Growth I"),
      outfit = "Magnus Gene Drive II",
      slot = "genedrive",
   },
   {
      name = _("Weapon Organ Growth I"),
      outfit = {
         "Tentacle Organ II",
         "Tentacle Organ II",
      },
      slot = {
         "rightweap",
         "leftweap",
      },
   },
   {
      name = _("Cerebrum Growth I"),
      outfit = "Immane Cerebrum II",
      slot = "brain",
   },
   {
      name = _("Cortex Growth II"),
      outfit = "Immanis Cortex III",
      slot = "shell",
   },
   {
      name = _("Weapon Organ Growth II"),
      outfit = {
         "Tentacle Organ III",
         "Tentacle Organ III",
      },
      slot = {
         "rightweap",
         "leftweap",
      },
   },
   {
      name = _("Gene Drive Growth II"),
      outfit = "Magnus Gene Drive III",
      slot = "genedrive",
   },
   {
      name = _("Cerebrum Growth II"),
      outfit = "Immane Cerebrum III",
      slot = "brain",
   },
   {
      name = _("Weapon Organ Growth III"),
      outfit = {
         "Tentacle Organ IV",
         "Tentacle Organ IV",
      },
      slot = {
         "rightweap",
         "leftweap",
      },
   },
}

-- Revenant (Corvette) intrinsics
intrinsics["Pirate Revenant"] = {
   maxstage = 5, -- 2 stages below
   {
      name = _("Innate"),
      outfit = {
         "Mediocre Cerebrum I",
         "Mediocris Gene Drive I",
         "Talon Organ I",
         "Talon Organ I",
      },
      slot = {
         "brain",
         "genedrive",
         "weap1",
         "weap2",
      },
   },
   {
      name = _("Gene Drive Growth I"),
      outfit = {
         "Mediocris Gene Drive II",
      },
      slot = {
         "genedrive",
      },
   },
   {
      name = _("Weapon Organ Growth"),
      outfit = {
         "Talon Organ II",
         "Talon Organ II",
      },
      slot = {
         "weap1",
         "weap2",
      },
   },
   {
      name = _("Cerebrum Growth"),
      outfit = {
         "Mediocre Cerebrum II",
      },
      slot = {
         "brain",
      },
   },
   {
      name = _("Gene Drive Growth II"),
      outfit = {
         "Mediocris Gene Drive III",
      },
      slot = {
         "genedrive",
      },
   },
}

return intrinsics
