local sokoban = require "minigames.sokoban.core"

love.load = sokoban.load
love.draw = sokoban.draw
love.update = sokoban.update
love.keypressed = sokoban.keypressed
love.mousemoved = sokoban.mousemoved
love.mousepressed = sokoban.mousepressed
love.mousereleased = sokoban.mousereleased
