return {
   fct            = faction.get("Frontier"),
   rep_max        = 30, -- Doesn't matter atm
   rep_max_var    = "_fcap_frontier",
   destroy_max    = 10,
   capture_max    = 10,
   disable_max    = 10,
   board_max      = 10,
   rep_max_tags   = {},
   hit_range      = 1,
}
