--[[
   Pure Lua implementation of special effects for use with missions
--]]
local luaspfx = {}

luaspfx.alert = require "luaspfx.alert"
luaspfx.blink = require "luaspfx.blink"
luaspfx.sfx   = require "luaspfx.sfx"
luaspfx.trail = require "luaspfx.trail"
luaspfx.pulse = require "luaspfx.pulse"
luaspfx.pheromones = require "luaspfx.pheromones"
luaspfx.explosion = require "luaspfx.explosion"
luaspfx.spacemine = require "luaspfx.spacemine"

return luaspfx
