--[[
<?xml version='1.0' encoding='utf8'?>
<event name="Quai IIb Astral Orchids">
 <location>land</location>
 <chance>100</chance>
 <spob>Quai IIb</spob>
 <tags>
  <tag>astral_orchids</tag>
 </tags>
</event>
--]]
require("common.astral_orchids").setup{ harvest_amount=5 }
