-- THIS FILE IS GENERATED BY utils/luacheckrc_extractor.py
-- ALL LOCAL CHANGES WILL BE LOST ON REGENERATION, SO DO NOT MODIFY PLEASE
local stds = {}
stds.ai = {
   read_globals = {
      ai = {
         fields = {
         accel = {},
         aim = {},
         board = {},
         brake = {},
         canHyperspace = {},
         canboard = {},
         careful_face = {},
         combat = {},
         dir = {},
         dist = {},
         dist2 = {},
         distress = {},
         dock = {},
         drift_facing = {},
         face = {},
         face_accurate = {},
         flyby_dist = {},
         follow_accurate = {},
         gatherablepos = {},
         getBoss = {},
         getenemy = {},
         getgatherable = {},
         getweapammo = {},
         getweaprange = {},
         getweaprangemin = {},
         getweapspeed = {},
         hasafterburner = {},
         hascannons = {},
         hasfighterbays = {},
         haslockon = {},
         hasprojectile = {},
         hasturrets = {},
         hostile = {},
         hyperspace = {},
         hyperspaceAbort = {},
         idir = {},
         iface = {},
         instantJump = {},
         isally = {},
         isbribed = {},
         isenemy = {},
         ismaxvel = {},
         isstopped = {},
         land = {},
         landspob = {},
         messages = {},
         minbrakedist = {},
         nearestpilot = {},
         nearestspob = {},
         nearhyptarget = {},
         outfitOffAll = {},
         pilot = {},
         popsubtask = {},
         poptask = {},
         pushsubtask = {},
         pushtask = {},
         refuel = {},
         reldps = {},
         relhp = {},
         relsize = {},
         relvel = {},
         rndhyptarget = {},
         rndpilot = {},
         rndspob = {},
         scandone = {},
         set_shoot_indicator = {},
         setasterotarget = {},
         setcredits = {},
         sethyptarget = {},
         settarget = {},
         settimer = {},
         shoot_indicator = {},
         spobfrompos = {},
         stealth = {},
         stop = {},
         subtaskdata = {},
         subtaskname = {},
         taskdata = {},
         taskname = {},
         timeup = {},
         turn = {},
         weapset = {},
         },
      },
   }
}
stds.asteroid = {
   read_globals = {
      asteroid = {
         fields = {
         __eq = {},
         __tostring = {},
         alertRange = {},
         armour = {},
         exists = {},
         field = {},
         get = {},
         getAll = {},
         materials = {},
         pos = {},
         scanned = {},
         setArmour = {},
         setPos = {},
         setState = {},
         setTimer = {},
         setVel = {},
         state = {},
         timer = {},
         vel = {},
         },
      },
   }
}
stds.audio = {
   read_globals = {
      audio = {
         fields = {
         __eq = {},
         __gc = {},
         __tostring = {},
         clone = {},
         getAttenuationDistances = {},
         getDuration = {},
         getPitch = {},
         getPosition = {},
         getRolloff = {},
         getVelocity = {},
         getVolume = {},
         isLooping = {},
         isPaused = {},
         isStopped = {},
         new = {},
         pause = {},
         play = {},
         rewind = {},
         seek = {},
         setAirAbsorption = {},
         setAttenuationDistances = {},
         setDopplerFactor = {},
         setEffect = {},
         setEffectData = {},
         setGlobalEffect = {},
         setLooping = {},
         setPitch = {},
         setPosition = {},
         setRelative = {},
         setRolloff = {},
         setSpeedOfSound = {},
         setVelocity = {},
         setVolume = {},
         soundPlay = {},
         stop = {},
         tell = {},
         },
      },
   }
}
stds.audiodata = {
   read_globals = {
      audiodata = {
         fields = {
         __tostring = {},
         new = {},
         },
      },
   }
}
stds.bkg = {
   read_globals = {
      bkg = {
         fields = {
         clear = {},
         image = {},
         },
      },
   }
}
stds.camera = {
   read_globals = {
      camera = {
         fields = {
         get = {},
         getZoom = {},
         pos = {},
         set = {},
         setZoom = {},
         shake = {},
         },
      },
   }
}
stds.canvas = {
   read_globals = {
      canvas = {
         fields = {
         __eq = {},
         __gc = {},
         clear = {},
         dims = {},
         getTex = {},
         new = {},
         set = {},
         },
      },
   }
}
stds.cli = {
   read_globals = {
      cli = {
         fields = {
         },
      },
   }
}
stds.colour = {
   read_globals = {
      colour = {
         fields = {
         __eq = {},
         __tostring = {},
         alpha = {},
         hsv = {},
         hsv_to_rgb = {},
         new = {},
         new_hsv = {},
         new_named = {},
         rgb = {},
         rgb_to_hsv = {},
         rgba = {},
         set_alpha = {},
         set_hsv = {},
         set_rgb = {},
         to_gamma = {},
         to_linear = {},
         },
      },
   }
}
stds.commodity = {
   read_globals = {
      commodity = {
         fields = {
         __eq = {},
         __tostring = {},
         canBuy = {},
         canSell = {},
         description = {},
         flags = {},
         get = {},
         getStandard = {},
         icon = {},
         illegality = {},
         illegalto = {},
         name = {},
         nameRaw = {},
         new = {},
         price = {},
         priceAt = {},
         priceAtTime = {},
         },
      },
   }
}
stds.data = {
   read_globals = {
      data = {
         fields = {
         __eq = {},
         __gc = {},
         addWeighted = {},
         convolve2d = {},
         get = {},
         getSize = {},
         getString = {},
         new = {},
         paste = {},
         set = {},
         },
      },
   }
}
stds.diff = {
   read_globals = {
      diff = {
         fields = {
         apply = {},
         isApplied = {},
         remove = {},
         },
      },
   }
}
stds.evt = {
   read_globals = {
      evt = {
         fields = {
         claim = {},
         finish = {},
         npcAdd = {},
         npcRm = {},
         save = {},
         },
      },
   }
}
stds.faction = {
   read_globals = {
      faction = {
         fields = {
         __eq = {},
         __tostring = {},
         allies = {},
         applyLocalThreshold = {},
         areAllies = {},
         areEnemies = {},
         areNeutral = {},
         colour = {},
         dynAdd = {},
         dynAlly = {},
         dynEnemy = {},
         enemies = {},
         exists = {},
         get = {},
         getAll = {},
         hit = {},
         hitTest = {},
         invisible = {},
         known = {},
         logo = {},
         longname = {},
         name = {},
         nameRaw = {},
         player = {},
         reputationDefault = {},
         reputationGlobal = {},
         reputationOverride = {},
         reputationText = {},
         setKnown = {},
         setReputationGlobal = {},
         setReputationOverride = {},
         static = {},
         tags = {},
         usesHiddenJumps = {},
         },
      },
   }
}
stds.file = {
   read_globals = {
      file = {
         fields = {
         __eq = {},
         __gc = {},
         close = {},
         enumerate = {},
         filetype = {},
         from_string = {},
         getFilename = {},
         getMode = {},
         getSize = {},
         isOpen = {},
         mkdir = {},
         new = {},
         open = {},
         read = {},
         remove = {},
         seek = {},
         write = {},
         },
      },
   }
}
stds.font = {
   read_globals = {
      font = {
         fields = {
         __eq = {},
         __gc = {},
         addFallback = {},
         height = {},
         new = {},
         setFilter = {},
         width = {},
         },
      },
   }
}
stds.gfx = {
   read_globals = {
      gfx = {
         fields = {
         clearDepth = {},
         dim = {},
         fontSize = {},
         glVersion = {},
         lightAmbient = {},
         lightAmbientGet = {},
         lightIntensity = {},
         lightIntensityGet = {},
         print = {},
         printDim = {},
         printH = {},
         printRestoreClear = {},
         printRestoreLast = {},
         printText = {},
         printf = {},
         printfDim = {},
         printfEnd = {},
         printfWrap = {},
         renderCircle = {},
         renderCircleH = {},
         renderLinesH = {},
         renderRect = {},
         renderRectH = {},
         renderTex = {},
         renderTexH = {},
         renderTexRaw = {},
         renderTexScale = {},
         screencoords = {},
         screenshot = {},
         setBlendMode = {},
         setBlendState = {},
         setScissor = {},
         },
      },
   }
}
stds.gui = {
   read_globals = {
      gui = {
         fields = {
         cameraOffset = {},
         fpsPos = {},
         mesgInit = {},
         mouseClickEnable = {},
         mouseMoveEnable = {},
         omsgInit = {},
         osdInit = {},
         radarInit = {},
         radarRender = {},
         setMapOverlayBounds = {},
         targetPilotGFX = {},
         targetSpobGFX = {},
         viewport = {},
         },
      },
   }
}
stds.hook = {
   read_globals = {
      hook = {
         fields = {
         asteroid_scan = {},
         board = {},
         comm_buy = {},
         comm_jettison = {},
         comm_sell = {},
         custom = {},
         date = {},
         discover = {},
         enter = {},
         equip = {},
         event_done = {},
         gather = {},
         hail = {},
         hail_spob = {},
         info = {},
         input = {},
         jumpin = {},
         jumpout = {},
         land = {},
         load = {},
         mission_done = {},
         mouse = {},
         outfit_buy = {},
         outfit_sell = {},
         pay = {},
         pilot = {},
         renderbg = {},
         renderfg = {},
         rendertop = {},
         rm = {},
         safe = {},
         ship_buy = {},
         ship_sell = {},
         ship_swap = {},
         standing = {},
         takeoff = {},
         timer = {},
         timerClear = {},
         update = {},
         },
      },
   }
}
stds.jump = {
   read_globals = {
      jump = {
         fields = {
         __eq = {},
         __tostring = {},
         angle = {},
         dest = {},
         exitonly = {},
         get = {},
         hidden = {},
         hide = {},
         jumpDist = {},
         known = {},
         pos = {},
         radius = {},
         reverse = {},
         setKnown = {},
         system = {},
         },
      },
   }
}
stds.linopt = {
   read_globals = {
      linopt = {
         fields = {
         __eq = {},
         __gc = {},
         add_cols = {},
         add_rows = {},
         load_matrix = {},
         new = {},
         read_problem = {},
         set_col = {},
         set_row = {},
         size = {},
         solve = {},
         write_problem = {},
         },
      },
   }
}
stds.misn = {
   read_globals = {
      misn = {
         fields = {
         accept = {},
         cargoAdd = {},
         cargoJet = {},
         cargoRm = {},
         claim = {},
         computerRefresh = {},
         factions = {},
         finish = {},
         markerAdd = {},
         markerMove = {},
         markerRm = {},
         npcAdd = {},
         npcRm = {},
         osdActive = {},
         osdCreate = {},
         osdDestroy = {},
         osdGet = {},
         osdGetActive = {},
         osdHide = {},
         osdSetHide = {},
         setDesc = {},
         setDistance = {},
         setFaction = {},
         setIllegal = {},
         setNPC = {},
         setReward = {},
         setTitle = {},
         },
      },
   }
}
stds.munition = {
   read_globals = {
      munition = {
         fields = {
         __eq = {},
         __tostring = {},
         clear = {},
         exists = {},
         faction = {},
         getAll = {},
         getInrange = {},
         outfit = {},
         parent = {},
         pos = {},
         strength = {},
         strengthSet = {},
         target = {},
         vel = {},
         },
      },
   }
}
stds.music = {
   read_globals = {
      music = {
         fields = {
         choose = {},
         current = {},
         getVolume = {},
         isPlaying = {},
         pause = {},
         play = {},
         stop = {},
         },
      },
   }
}
stds.naev = {
   read_globals = {
      naev = {
         fields = {
         cache = {},
         claimTest = {},
         clock = {},
         conf = {},
         confSet = {},
         date = {},
         debugCollisions = {},
         debugTrails = {},
         difficulty = {},
         difficultyLevel = {},
         eventReload = {},
         eventStart = {},
         fps = {},
         hasTextInput = {},
         isPaused = {},
         isSimulation = {},
         keyDisableAll = {},
         keyEnable = {},
         keyEnableAll = {},
         keyGet = {},
         language = {},
         lastplayed = {},
         menuInfo = {},
         menuSmall = {},
         missionList = {},
         missionReload = {},
         missionStart = {},
         missionTest = {},
         pause = {},
         plugins = {},
         quadtreeParams = {},
         setTextInput = {},
         shadersReload = {},
         shipstats = {},
         supported_image_ext = {},
         ticks = {},
         ticksGame = {},
         trigger = {},
         unit = {},
         unpause = {},
         version = {},
         versionTest = {},
         },
      },
   }
}
stds.news = {
   read_globals = {
      news = {
         fields = {
         __eq = {},
         add = {},
         bind = {},
         date = {},
         desc = {},
         faction = {},
         get = {},
         rm = {},
         title = {},
         },
      },
   }
}
stds.outfit = {
   read_globals = {
      outfit = {
         fields = {
         __eq = {},
         __tostring = {},
         cpu = {},
         description = {},
         exists = {},
         friendlyfire = {},
         get = {},
         getAll = {},
         icon = {},
         illegality = {},
         known = {},
         licence = {},
         license = {},
         limit = {},
         mass = {},
         missAsteroids = {},
         missShips = {},
         name = {},
         nameRaw = {},
         pointdefence = {},
         pointdefense = {},
         price = {},
         shipstat = {},
         shortname = {},
         slot = {},
         slotExtra = {},
         specificstats = {},
         summary = {},
         tags = {},
         toggleable = {},
         type = {},
         typeBroad = {},
         unique = {},
         weapstats = {},
         },
      },
   }
}
stds.pilot = {
   read_globals = {
      pilot = {
         fields = {
         __eq = {},
         __tostring = {},
         accel = {},
         actives = {},
         add = {},
         addEnergy = {},
         addHealth = {},
         ai = {},
         ainame = {},
         areAllies = {},
         areEnemies = {},
         armour = {},
         attack = {},
         autoweap = {},
         board = {},
         brake = {},
         broadcast = {},
         calcStats = {},
         canHyperspace = {},
         canSpawn = {},
         cargoAdd = {},
         cargoFree = {},
         cargoHas = {},
         cargoJet = {},
         cargoList = {},
         cargoRm = {},
         changeAI = {},
         choosePoint = {},
         clear = {},
         clearSelect = {},
         clone = {},
         collisionTest = {},
         colour = {},
         colourChar = {},
         comm = {},
         control = {},
         cooldown = {},
         cooldownCycle = {},
         credits = {},
         damage = {},
         destealth = {},
         detectedDistance = {},
         dir = {},
         disabled = {},
         distress = {},
         dockslot = {},
         effectAdd = {},
         effectClear = {},
         effectHas = {},
         effectRm = {},
         effects = {},
         energy = {},
         exists = {},
         explode = {},
         face = {},
         faction = {},
         fillAmmo = {},
         flags = {},
         follow = {},
         followers = {},
         fuel = {},
         gather = {},
         get = {},
         getAllies = {},
         getEnemies = {},
         getInrange = {},
         getVisible = {},
         hailPlayer = {},
         hasIllegal = {},
         health = {},
         hookClear = {},
         hostile = {},
         hyperspace = {},
         id = {},
         idle = {},
         inrange = {},
         intrinsicGet = {},
         intrinsicReset = {},
         intrinsicSet = {},
         invincible = {},
         isStopped = {},
         jamLockons = {},
         kill = {},
         knockback = {},
         land = {},
         leader = {},
         lockon = {},
         mass = {},
         memory = {},
         mothership = {},
         moveto = {},
         msg = {},
         name = {},
         nav = {},
         navJump = {},
         navJumpSet = {},
         navSpob = {},
         navSpobSet = {},
         outfitAdd = {},
         outfitAddIntrinsic = {},
         outfitAddSlot = {},
         outfitGet = {},
         outfitHasSlot = {},
         outfitInitSlot = {},
         outfitMessageSlot = {},
         outfitReady = {},
         outfitRm = {},
         outfitRmIntrinsic = {},
         outfitRmSlot = {},
         outfitSlot = {},
         outfitToggle = {},
         outfits = {},
         outfitsEquip = {},
         outfitsList = {},
         outfitsReset = {},
         points = {},
         poptask = {},
         pos = {},
         pushtask = {},
         radius = {},
         refuel = {},
         rename = {},
         render = {},
         renderComm = {},
         renderTo = {},
         reputation = {},
         rm = {},
         runaway = {},
         scandone = {},
         setActiveBoard = {},
         setBribed = {},
         setCooldown = {},
         setDir = {},
         setDisable = {},
         setEnergy = {},
         setFaction = {},
         setFriendly = {},
         setFuel = {},
         setHealth = {},
         setHealthAbs = {},
         setHide = {},
         setHilight = {},
         setHostile = {},
         setInvincPlayer = {},
         setInvincible = {},
         setInvisible = {},
         setLeader = {},
         setNoBoard = {},
         setNoClear = {},
         setNoDeath = {},
         setNoDisable = {},
         setNoJump = {},
         setNoLand = {},
         setNoRender = {},
         setPos = {},
         setSpeedLimit = {},
         setTarget = {},
         setTargetAsteroid = {},
         setVel = {},
         setVisible = {},
         setVisplayer = {},
         shield = {},
         ship = {},
         shipMemory = {},
         shippropGet = {},
         shippropReset = {},
         shippropSet = {},
         shipstat = {},
         shipvarPeek = {},
         shipvarPop = {},
         shipvarPush = {},
         signature = {},
         spaceworthy = {},
         speed = {},
         speedMax = {},
         stats = {},
         stealth = {},
         stealthRange = {},
         stress = {},
         target = {},
         targetAsteroid = {},
         task = {},
         taskClear = {},
         taskdata = {},
         taskname = {},
         taskstack = {},
         toggleSpawn = {},
         tryStealth = {},
         turn = {},
         vel = {},
         weapset = {},
         weapsetAdd = {},
         weapsetAddType = {},
         weapsetAmmo = {},
         weapsetAuto = {},
         weapsetCleanup = {},
         weapsetList = {},
         weapsetRm = {},
         weapsetSetInrange = {},
         weapsetType = {},
         withPlayer = {},
         worth = {},
         },
      },
   }
}
stds.pilotoutfit = {
   read_globals = {
      pilotoutfit = {
         fields = {
         clear = {},
         id = {},
         mount = {},
         munition = {},
         outfit = {},
         progress = {},
         set = {},
         shoot = {},
         slot = {},
         state = {},
         },
      },
   }
}
stds.player = {
   read_globals = {
      player = {
         fields = {
         allowSave = {},
         autonav = {},
         autonavAbort = {},
         autonavDest = {},
         autonavEnd = {},
         autonavReset = {},
         autonavRoute = {},
         autonavSetPos = {},
         autonavSetSpeed = {},
         autonavSetTarget = {},
         canDiscover = {},
         chapter = {},
         chapterSet = {},
         cinematics = {},
         cinematicsCheck = {},
         commClose = {},
         commOpen = {},
         credits = {},
         damageSPFX = {},
         dt_default = {},
         dt_mod = {},
         evtActive = {},
         evtDone = {},
         evtDoneList = {},
         fleetCapacity = {},
         fleetCapacitySet = {},
         fleetCargoAdd = {},
         fleetCargoFree = {},
         fleetCargoJet = {},
         fleetCargoList = {},
         fleetCargoMissionFree = {},
         fleetCargoOwned = {},
         fleetCargoRm = {},
         fleetCargoUsed = {},
         fleetList = {},
         fuel = {},
         gameover = {},
         gui = {},
         guiList = {},
         guiSet = {},
         hasLicence = {},
         hasLicense = {},
         infoButtonRegister = {},
         infoButtonUnregister = {},
         inventory = {},
         inventoryAdd = {},
         inventoryOwned = {},
         inventoryRm = {},
         isLanded = {},
         jumps = {},
         land = {},
         landAllow = {},
         landWindow = {},
         merchantOutfitName = {},
         misnActive = {},
         misnDone = {},
         misnDoneList = {},
         missions = {},
         msg = {},
         msgClear = {},
         msgToggle = {},
         name = {},
         omsgAdd = {},
         omsgChange = {},
         omsgRm = {},
         outfitAdd = {},
         outfitNum = {},
         outfitRm = {},
         outfits = {},
         pay = {},
         pilot = {},
         pos = {},
         refuel = {},
         save = {},
         saveBackup = {},
         screenshot = {},
         setSpeed = {},
         ship = {},
         shipAdd = {},
         shipDeploy = {},
         shipMetadata = {},
         shipOutfits = {},
         shipSwap = {},
         ships = {},
         shipvarPeek = {},
         shipvarPop = {},
         shipvarPush = {},
         speed = {},
         start = {},
         takeoff = {},
         teleport = {},
         tryBoard = {},
         tryLand = {},
         unboard = {},
         wealth = {},
         },
      },
   }
}
stds.rnd = {
   read_globals = {
      rnd = {
         fields = {
         angle = {},
         permutation = {},
         rnd = {},
         sigma = {},
         threesigma = {},
         twosigma = {},
         uniform = {},
         },
      },
   }
}
stds.safelanes = {
   read_globals = {
      safelanes = {
         fields = {
         get = {},
         },
      },
   }
}
stds.shader = {
   read_globals = {
      shader = {
         fields = {
         __eq = {},
         __gc = {},
         addPPShader = {},
         hasUniform = {},
         new = {},
         rmPPShader = {},
         send = {},
         sendRaw = {},
         },
      },
   }
}
stds.ship = {
   read_globals = {
      ship = {
         fields = {
         __eq = {},
         __tostring = {},
         armour = {},
         baseType = {},
         cargo = {},
         class = {},
         classDisplay = {},
         cpu = {},
         crew = {},
         description = {},
         dims = {},
         fabricator = {},
         faction = {},
         fitsSlot = {},
         fuelConsumption = {},
         get = {},
         getAll = {},
         getSlots = {},
         gfx = {},
         gfxComm = {},
         gfxPath = {},
         gfxStore = {},
         inherits = {},
         known = {},
         license = {},
         mass = {},
         name = {},
         nameRaw = {},
         points = {},
         price = {},
         render = {},
         screenSize = {},
         shipstat = {},
         shipstatDesc = {},
         size = {},
         slots = {},
         tags = {},
         time_mod = {},
         },
      },
   }
}
stds.shiplog = {
   read_globals = {
      shiplog = {
         fields = {
         append = {},
         create = {},
         },
      },
   }
}
stds.spfx = {
   read_globals = {
      spfx = {
         fields = {
         __eq = {},
         __gc = {},
         data = {},
         debris = {},
         getAll = {},
         nebulaColour = {},
         new = {},
         pos = {},
         rm = {},
         setPos = {},
         setVel = {},
         sfx = {},
         vel = {},
         },
      },
   }
}
stds.spob = {
   read_globals = {
      spob = {
         fields = {
         __eq = {},
         __tostring = {},
         blackmarket = {},
         canLand = {},
         class = {},
         classLong = {},
         colour = {},
         colourChar = {},
         commoditiesSold = {},
         cur = {},
         exists = {},
         faction = {},
         flags = {},
         get = {},
         getAll = {},
         getLandAllow = {},
         getLandDeny = {},
         getLandable = {},
         getS = {},
         gfxComm = {},
         gfxExterior = {},
         gfxExteriorPath = {},
         gfxSpace = {},
         hostile = {},
         known = {},
         landAllow = {},
         landDeny = {},
         name = {},
         nameRaw = {},
         outfitsSold = {},
         population = {},
         pos = {},
         radius = {},
         recordCommodityPriceAtTime = {},
         reputation = {},
         services = {},
         setHostile = {},
         setKnown = {},
         shipsSold = {},
         system = {},
         tags = {},
         },
      },
   }
}
stds.system = {
   read_globals = {
      system = {
         fields = {
         __eq = {},
         __tostring = {},
         addGatherable = {},
         adjacentSystems = {},
         asteroidFields = {},
         background = {},
         cur = {},
         faction = {},
         get = {},
         getAll = {},
         hidden = {},
         interference = {},
         jumpDist = {},
         jumpPath = {},
         jumps = {},
         known = {},
         markerAdd = {},
         markerClear = {},
         markerRm = {},
         name = {},
         nameRaw = {},
         nebula = {},
         pos = {},
         presence = {},
         presences = {},
         radius = {},
         reputation = {},
         setHidden = {},
         setKnown = {},
         setReputation = {},
         spobs = {},
         tags = {},
         waypoints = {},
         },
      },
   }
}
stds.tex = {
   read_globals = {
      tex = {
         fields = {
         __gc = {},
         dim = {},
         new = {},
         open = {},
         readData = {},
         setFilter = {},
         setWrap = {},
         spriteFromDir = {},
         sprites = {},
         writeData = {},
         },
      },
   }
}
stds.time = {
   read_globals = {
      time = {
         fields = {
         __add = {},
         __eq = {},
         __le = {},
         __lt = {},
         __sub = {},
         __tostring = {},
         add = {},
         fromnumber = {},
         get = {},
         inc = {},
         new = {},
         str = {},
         sub = {},
         tonumber = {},
         },
      },
   }
}
stds.tk = {
   read_globals = {
      tk = {
         fields = {
         choice = {},
         custom = {},
         customDone = {},
         customFullscreen = {},
         customRename = {},
         customResize = {},
         customSize = {},
         input = {},
         isOpen = {},
         list = {},
         merchantOutfit = {},
         msg = {},
         query = {},
         refresh = {},
         yesno = {},
         },
      },
   }
}
stds.transform = {
   read_globals = {
      transform = {
         fields = {
         __eq = {},
         __mul = {},
         __tostring = {},
         applyDim = {},
         applyPoint = {},
         get = {},
         new = {},
         ortho = {},
         rotate2d = {},
         scale = {},
         set = {},
         translate = {},
         },
      },
   }
}
stds.var = {
   read_globals = {
      var = {
         fields = {
         peek = {},
         pop = {},
         push = {},
         },
      },
   }
}
stds.vec2 = {
   read_globals = {
      vec2 = {
         fields = {
         __add = {},
         __div = {},
         __mul = {},
         __sub = {},
         __tostring = {},
         __unm = {},
         add = {},
         angle = {},
         clone = {},
         collideCircleLine = {},
         collideLineLine = {},
         copy = {},
         cross = {},
         dist = {},
         dist2 = {},
         div = {},
         dot = {},
         get = {},
         mod = {},
         mul = {},
         new = {},
         newP = {},
         normalize = {},
         polar = {},
         set = {},
         setP = {},
         sub = {},
         },
      },
   }
}
stds.naev.read_globals.naev.fields.ai = stds.ai.read_globals.ai
stds.naev.read_globals.naev.fields.asteroid = stds.asteroid.read_globals.asteroid
stds.naev.read_globals.naev.fields.audio = stds.audio.read_globals.audio
stds.naev.read_globals.naev.fields.audiodata = stds.audiodata.read_globals.audiodata
stds.naev.read_globals.naev.fields.bkg = stds.bkg.read_globals.bkg
stds.naev.read_globals.naev.fields.camera = stds.camera.read_globals.camera
stds.naev.read_globals.naev.fields.canvas = stds.canvas.read_globals.canvas
stds.naev.read_globals.naev.fields.cli = stds.cli.read_globals.cli
stds.naev.read_globals.naev.fields.colour = stds.colour.read_globals.colour
stds.naev.read_globals.naev.fields.commodity = stds.commodity.read_globals.commodity
stds.naev.read_globals.naev.fields.data = stds.data.read_globals.data
stds.naev.read_globals.naev.fields.diff = stds.diff.read_globals.diff
stds.naev.read_globals.naev.fields.evt = stds.evt.read_globals.evt
stds.naev.read_globals.naev.fields.faction = stds.faction.read_globals.faction
stds.naev.read_globals.naev.fields.file = stds.file.read_globals.file
stds.naev.read_globals.naev.fields.font = stds.font.read_globals.font
stds.naev.read_globals.naev.fields.gfx = stds.gfx.read_globals.gfx
stds.naev.read_globals.naev.fields.gui = stds.gui.read_globals.gui
stds.naev.read_globals.naev.fields.hook = stds.hook.read_globals.hook
stds.naev.read_globals.naev.fields.jump = stds.jump.read_globals.jump
stds.naev.read_globals.naev.fields.linopt = stds.linopt.read_globals.linopt
stds.naev.read_globals.naev.fields.misn = stds.misn.read_globals.misn
stds.naev.read_globals.naev.fields.munition = stds.munition.read_globals.munition
stds.naev.read_globals.naev.fields.music = stds.music.read_globals.music
stds.naev.read_globals.naev.fields.news = stds.news.read_globals.news
stds.naev.read_globals.naev.fields.outfit = stds.outfit.read_globals.outfit
stds.naev.read_globals.naev.fields.pilot = stds.pilot.read_globals.pilot
stds.naev.read_globals.naev.fields.pilotoutfit = stds.pilotoutfit.read_globals.pilotoutfit
stds.naev.read_globals.naev.fields.player = stds.player.read_globals.player
stds.naev.read_globals.naev.fields.rnd = stds.rnd.read_globals.rnd
stds.naev.read_globals.naev.fields.safelanes = stds.safelanes.read_globals.safelanes
stds.naev.read_globals.naev.fields.shader = stds.shader.read_globals.shader
stds.naev.read_globals.naev.fields.ship = stds.ship.read_globals.ship
stds.naev.read_globals.naev.fields.shiplog = stds.shiplog.read_globals.shiplog
stds.naev.read_globals.naev.fields.spfx = stds.spfx.read_globals.spfx
stds.naev.read_globals.naev.fields.spob = stds.spob.read_globals.spob
stds.naev.read_globals.naev.fields.system = stds.system.read_globals.system
stds.naev.read_globals.naev.fields.tex = stds.tex.read_globals.tex
stds.naev.read_globals.naev.fields.time = stds.time.read_globals.time
stds.naev.read_globals.naev.fields.tk = stds.tk.read_globals.tk
stds.naev.read_globals.naev.fields.transform = stds.transform.read_globals.transform
stds.naev.read_globals.naev.fields.var = stds.var.read_globals.var
stds.naev.read_globals.naev.fields.vec2 = stds.vec2.read_globals.vec2
return stds
