local luaspob = require "spob.lua.lib.spob"

luaspob.setup{
   std_land = -20,
   std_bribe = -50,
   msg_granted = {
      _([["Welcome to the Jade Court."]]),
      _([["Landing permission approved."]]),
   }
}
