#!/usr/bin/env python3
# Auto-generated by Meson during the build
import os
import sys
import subprocess

BINARY_PATH = "@pluginmgr_cli_bin@"

def main():
   if not os.path.exists(BINARY_PATH):
      print(f"Error: {BINARY_PATH} not found. Did you run 'meson compile pluginmgr-cli?'?")
      sys.exit(1)
   os.execvp(BINARY_PATH, [BINARY_PATH, *sys.argv[1:]])

if __name__ == "__main__":
   main()
