/*
 * See Licensing and Copyright notice in naev.h
 */
#pragma once

void map_system_open( int sys_selected );
int  map_system_init( void );
int  map_system_load( void );
void map_system_exit( void );
