/*
 * See Licensing and Copyright notice in naev.h
 */
#pragma once

/*
 * Init/exit.
 */
int  cli_init( void );
void cli_exit( void );

/*
 * Misc.
 */
void cli_open( void );
int  cli_isOpen( void );
void cli_printCoreString( const char *s, int escape );
