return {
   fct            = faction.get("Proteron"),
   rep_max        = 70, --30, -- TODO lower as missions get added
   rep_max_var    = "_fcap_proteron",
   secondary_default = 0, -- Unaffected by secondary hits, oblivious to the universe (for now...)
   rep_max_tags   = {},
}
