/*
 * See Licensing and Copyright notice in naev.h
 */
#pragma once

void debris_cleanup( void );
void debris_add( double mass, double rad, double px, double py, double vx,
                 double vy );
