/*
 * See Licensing and Copyright notice in naev.h
 */
#pragma once

#include "tk/widget/button.h"
#include "tk/widget/checkbox.h"
#include "tk/widget/cust.h"
#include "tk/widget/fader.h"
#include "tk/widget/image.h"
#include "tk/widget/imagearray.h"
#include "tk/widget/input.h"
#include "tk/widget/list.h"
#include "tk/widget/rect.h"
#include "tk/widget/tabwin.h"
#include "tk/widget/text.h"
