// clang-format off
/*
  Simple DirectMedia Layer
  Copyright (C) 1997-2013 Sam Lantinga <slouken@libsdl.org>

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgement in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/
/*
 * This is just taken straight from src/events/SDL_keyboard.c from the SDL2
 * source code, and then regex'd to have all strings wrapped in p_("keyname",).
 * It is not meant to be compiled, only for gettext.
 */
static const char *SDL_scancode_names[SDL_NUM_SCANCODES] = {
    NULL, NULL, NULL, NULL,
    p_("keyname","A"),
    p_("keyname","B"),
    p_("keyname","C"),
    p_("keyname","D"),
    p_("keyname","E"),
    p_("keyname","F"),
    p_("keyname","G"),
    p_("keyname","H"),
    p_("keyname","I"),
    p_("keyname","J"),
    p_("keyname","K"),
    p_("keyname","L"),
    p_("keyname","M"),
    p_("keyname","N"),
    p_("keyname","O"),
    p_("keyname","P"),
    p_("keyname","Q"),
    p_("keyname","R"),
    p_("keyname","S"),
    p_("keyname","T"),
    p_("keyname","U"),
    p_("keyname","V"),
    p_("keyname","W"),
    p_("keyname","X"),
    p_("keyname","Y"),
    p_("keyname","Z"),
    p_("keyname","1"),
    p_("keyname","2"),
    p_("keyname","3"),
    p_("keyname","4"),
    p_("keyname","5"),
    p_("keyname","6"),
    p_("keyname","7"),
    p_("keyname","8"),
    p_("keyname","9"),
    p_("keyname","0"),
    p_("keyname","Return"),
    p_("keyname","Escape"),
    p_("keyname","Backspace"),
    p_("keyname","Tab"),
    p_("keyname","Space"),
    p_("keyname","-"),
    p_("keyname","="),
    p_("keyname","["),
    p_("keyname","]"),
    p_("keyname","\\"),
    p_("keyname","#"),
    p_("keyname",";"),
    p_("keyname","'"),
    p_("keyname","`"),
    p_("keyname","("),
    p_("keyname",")"),
    p_("keyname","."),
    p_("keyname","/"),
    p_("keyname","CapsLock"),
    p_("keyname","F1"),
    p_("keyname","F2"),
    p_("keyname","F3"),
    p_("keyname","F4"),
    p_("keyname","F5"),
    p_("keyname","F6"),
    p_("keyname","F7"),
    p_("keyname","F8"),
    p_("keyname","F9"),
    p_("keyname","F10"),
    p_("keyname","F11"),
    p_("keyname","F12"),
    p_("keyname","PrintScreen"),
    p_("keyname","ScrollLock"),
    p_("keyname","Pause"),
    p_("keyname","Insert"),
    p_("keyname","Home"),
    p_("keyname","PageUp"),
    p_("keyname","Delete"),
    p_("keyname","End"),
    p_("keyname","PageDown"),
    p_("keyname","Right"),
    p_("keyname","Left"),
    p_("keyname","Down"),
    p_("keyname","Up"),
    p_("keyname","Numlock"),
    p_("keyname","Keypad /"),
    p_("keyname","Keypad *"),
    p_("keyname","Keypad -"),
    p_("keyname","Keypad +"),
    p_("keyname","Keypad Enter"),
    p_("keyname","Keypad 1"),
    p_("keyname","Keypad 2"),
    p_("keyname","Keypad 3"),
    p_("keyname","Keypad 4"),
    p_("keyname","Keypad 5"),
    p_("keyname","Keypad 6"),
    p_("keyname","Keypad 7"),
    p_("keyname","Keypad 8"),
    p_("keyname","Keypad 9"),
    p_("keyname","Keypad 0"),
    p_("keyname","Keypad ."),
    NULL,
    p_("keyname","Application"),
    p_("keyname","Power"),
    p_("keyname","Keypad ="),
    p_("keyname","F13"),
    p_("keyname","F14"),
    p_("keyname","F15"),
    p_("keyname","F16"),
    p_("keyname","F17"),
    p_("keyname","F18"),
    p_("keyname","F19"),
    p_("keyname","F20"),
    p_("keyname","F21"),
    p_("keyname","F22"),
    p_("keyname","F23"),
    p_("keyname","F24"),
    p_("keyname","Execute"),
    p_("keyname","Help"),
    p_("keyname","Menu"),
    p_("keyname","Select"),
    p_("keyname","Stop"),
    p_("keyname","Again"),
    p_("keyname","Undo"),
    p_("keyname","Cut"),
    p_("keyname","Copy"),
    p_("keyname","Paste"),
    p_("keyname","Find"),
    p_("keyname","Mute"),
    p_("keyname","VolumeUp"),
    p_("keyname","VolumeDown"),
    NULL, NULL, NULL,
    p_("keyname","Keypad ,"),
    p_("keyname","Keypad = (AS400)"),
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL,
    p_("keyname","AltErase"),
    p_("keyname","SysReq"),
    p_("keyname","Cancel"),
    p_("keyname","Clear"),
    p_("keyname","Prior"),
    p_("keyname","Return"),
    p_("keyname","Separator"),
    p_("keyname","Out"),
    p_("keyname","Oper"),
    p_("keyname","Clear / Again"),
    p_("keyname","CrSel"),
    p_("keyname","ExSel"),
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    p_("keyname","Keypad 00"),
    p_("keyname","Keypad 000"),
    p_("keyname","ThousandsSeparator"),
    p_("keyname","DecimalSeparator"),
    p_("keyname","CurrencyUnit"),
    p_("keyname","CurrencySubUnit"),
    p_("keyname","Keypad ("),
    p_("keyname","Keypad )"),
    p_("keyname","Keypad {"),
    p_("keyname","Keypad }"),
    p_("keyname","Keypad Tab"),
    p_("keyname","Keypad Backspace"),
    p_("keyname","Keypad A"),
    p_("keyname","Keypad B"),
    p_("keyname","Keypad C"),
    p_("keyname","Keypad D"),
    p_("keyname","Keypad E"),
    p_("keyname","Keypad F"),
    p_("keyname","Keypad XOR"),
    p_("keyname","Keypad ^"),
    p_("keyname","Keypad %"),
    p_("keyname","Keypad <"),
    p_("keyname","Keypad >"),
    p_("keyname","Keypad &"),
    p_("keyname","Keypad &&"),
    p_("keyname","Keypad |"),
    p_("keyname","Keypad ||"),
    p_("keyname","Keypad :"),
    p_("keyname","Keypad #"),
    p_("keyname","Keypad Space"),
    p_("keyname","Keypad @"),
    p_("keyname","Keypad !"),
    p_("keyname","Keypad MemStore"),
    p_("keyname","Keypad MemRecall"),
    p_("keyname","Keypad MemClear"),
    p_("keyname","Keypad MemAdd"),
    p_("keyname","Keypad MemSubtract"),
    p_("keyname","Keypad MemMultiply"),
    p_("keyname","Keypad MemDivide"),
    p_("keyname","Keypad +/-"),
    p_("keyname","Keypad Clear"),
    p_("keyname","Keypad ClearEntry"),
    p_("keyname","Keypad Binary"),
    p_("keyname","Keypad Octal"),
    p_("keyname","Keypad Decimal"),
    p_("keyname","Keypad Hexadecimal"),
    NULL, NULL,
    p_("keyname","Left Ctrl"),
    p_("keyname","Left Shift"),
    p_("keyname","Left Alt"),
    p_("keyname","Left GUI"),
    p_("keyname","Right Ctrl"),
    p_("keyname","Right Shift"),
    p_("keyname","Right Alt"),
    p_("keyname","Right GUI"),
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL,
    p_("keyname","ModeSwitch"),
    p_("keyname","AudioNext"),
    p_("keyname","AudioPrev"),
    p_("keyname","AudioStop"),
    p_("keyname","AudioPlay"),
    p_("keyname","AudioMute"),
    p_("keyname","MediaSelect"),
    p_("keyname","WWW"),
    p_("keyname","Mail"),
    p_("keyname","Calculator"),
    p_("keyname","Computer"),
    p_("keyname","AC Search"),
    p_("keyname","AC Home"),
    p_("keyname","AC Back"),
    p_("keyname","AC Forward"),
    p_("keyname","AC Stop"),
    p_("keyname","AC Refresh"),
    p_("keyname","AC Bookmarks"),
    p_("keyname","BrightnessDown"),
    p_("keyname","BrightnessUp"),
    p_("keyname","DisplaySwitch"),
    p_("keyname","KBDIllumToggle"),
    p_("keyname","KBDIllumDown"),
    p_("keyname","KBDIllumUp"),
    p_("keyname","Eject"),
    p_("keyname","Sleep"),
};
